Documentation
¶
Overview ¶
Package replay implements a replay movement sensor that can return motion data.
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ( // ErrEndOfDataset represents that the replay sensor has reached the end of the dataset. ErrEndOfDataset = errors.New("reached end of dataset") )
Functions ¶
This section is empty.
Types ¶
type Config ¶
type Config struct {
Source string `json:"source,omitempty"`
RobotID string `json:"robot_id,omitempty"`
LocationID string `json:"location_id,omitempty"`
OrganizationID string `json:"organization_id,omitempty"`
Interval TimeInterval `json:"time_interval,omitempty"`
BatchSize *uint64 `json:"batch_size,omitempty"`
APIKey string `json:"api_key,omitempty"`
APIKeyID string `json:"api_key_id,omitempty"`
}
Config describes how to configure the replay movement sensor.
type TimeInterval ¶
type TimeInterval struct {
Start string `json:"start,omitempty"`
End string `json:"end,omitempty"`
}
TimeInterval holds the start and end time used to filter data.
Click to show internal directories.
Click to hide internal directories.